from argparse import ArgumentParser
import subprocess


def has_conflict(branch_name):
    try:
        base_commit_sha = subprocess.check_output(['git', 'merge-base', 'HEAD', branch_name]).decode().strip()
        patch = subprocess.run(['git', 'format-patch', f'{base_commit_sha}..{branch_name}', '--stdout'], check=True, capture_output=True)
        apply_check = subprocess.run(['git', 'apply', '--3way', '--check', '-'],
                                    input=patch.stdout, capture_output=True)

        if "with conflicts" in apply_check.stderr.decode():
            return True
        else:
            return False
    
    except subprocess.CalledProcessError as e:
        raise Exception('Git command error', e)


def main():
    print('Running...')
    parser = ArgumentParser(prog='cli')
    parser.add_argument("-a","--all-branches",action="store_true",help="Check all remote branches against your current HEAD and staging")
    args = parser.parse_args()

    remote_branches = {'origin/master'}

    if args.all_branches:
        remote_branches = set(subprocess.check_output(["git", "branch", "-r"]).decode().split('\n'))
        remote_branches = {branch.strip() for branch in remote_branches if branch != ''}

    subprocess.check_output(["git", "fetch"])

    for branch_name in remote_branches:
        if has_conflict(branch_name):
            print(f"{branch_name} has a conflict with our branch!")


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Note: To use the 'upload' functionality of this file, you must install Twine:
#   $ pip install -r requirements.txt

import io
import os
import sys
from shutil import rmtree

from setuptools import find_packages, setup, Command
NAME = 'conflictwatch'
DESCRIPTION = 'Check all remote branches against your current HEAD and staging'
URL = 'https://github.com/alisternorris/conflictwatch'
EMAIL = 'hi@alisternorris.com'
AUTHOR = 'Alister Norris'
REQUIRES_PYTHON = '>=3.6.0'
VERSION = '0.0.2'
LICENSE = 'MIT'
REQUIRED = []

here = os.path.abspath(os.path.dirname(__file__))

with io.open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = '\n' + f.read()

about = {}
if not VERSION:
    with open(os.path.join(here, NAME, '__version__.py')) as f:
        exec(f.read(), about)
else:
    about['__version__'] = VERSION

setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    author=AUTHOR,
    author_email=EMAIL,
    packages=['conflictwatch'],
    python_requires=REQUIRES_PYTHON,
    url=URL,

    entry_points={
        'console_scripts': ['conflictwatch=conflictwatch.cli:main'],
    },
    install_requires=REQUIRED,
    include_package_data=True,
    license=LICENSE,
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
)
